/*
 * IPSec Passthrough Modifier: Chad Sheu, CharlesTU
 * Reference: kernel 2.6.20.15
 */  

//#include <linux/config.h>
#include <linux/types.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/netfilter.h>
#include <linux/if.h>

#include <net/netfilter/nf_nat.h>
#include <net/netfilter/nf_nat_rule.h>
#include <net/netfilter/nf_nat_protocol.h>

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Netfilter NAT protocol helper module for ESP");
MODULE_AUTHOR("Chad Sheu, CharlesTU");

#if 0
    #define DEBUGP(format, args...) printk(KERN_DEBUG "%s :%s : " format,__FILE__ ,__FUNCTION__ , ## args)
    #define DUMP_TUPLE_ESP(x) printk("%u.%u.%u.%u:0x%x -> %u.%u.%u.%u:0x%x\n", \
	    		                    NIPQUAD((x)->src.ip), ntohl((x)->src.u.esp.spi), \
		        	                NIPQUAD((x)->dst.ip), ntohl((x)->dst.u.esp.spi))
#else
    #define DUMP_TUPLE_ESP(x)
    #define DEBUGP(x, args...)
#endif

static int esp_in_range(const struct nf_conntrack_tuple *tuple,
			    enum nf_nat_manip_type manip_type,
			    const union nf_conntrack_man_proto *min,
			    const union nf_conntrack_man_proto *max)
{
	return 1;
}

static int esp_unique_tuple(struct nf_conntrack_tuple *tuple,
				const struct nf_nat_range *range,
				enum nf_nat_manip_type maniptype,
				const struct nf_conntrack *conntrack)
{
	u_int32_t min, i, range_size;
	u_int32_t key = 0, *spi = NULL;
	   
	DUMP_TUPLE_ESP(tuple);
	if (maniptype == IP_NAT_MANIP_SRC)
		spi = (u_int32_t *) &tuple->src.u.esp.spi;
	else
		spi = &tuple->dst.u.esp.spi;
	
	/*range is to do fast search*/
	if (! (range->flags & IP_NAT_RANGE_PROTO_SPECIFIED)) {
		min = 1;
		range_size = 0xffffffff;
	} 
	else {
		min = ntohl(range->min.esp.spi);
		range_size = ntohl(range->max.esp.spi) - min + 1;
	}

	for (i = 0; i < range_size; i++, key++) {
		*spi = htonl(min + key % range_size);
		if (!nf_nat_used_tuple(tuple, (const struct nf_conn *) conntrack))
			return 1;
	}
	return 0;
}

static int esp_manip_pkt(struct sk_buff **pskb, unsigned int iphdroff,
				const struct nf_conntrack_tuple *tuple,
				enum nf_nat_manip_type maniptype)
{   
	struct iphdr *iph = (struct iphdr *) ((*pskb)->data + iphdroff);

	struct esp_hdr *esph = (struct esp_hdr *)((u_int32_t *)iph+iph->ihl);
	u_int32_t *old_ip;
	u_int32_t *old_spi = NULL;
	u_int32_t sip = 0, dip = 0, ret = 0;
	
	return 1;
}

/* nat helper struct */
static struct nf_nat_protocol esp = {
	.name            = "esp",
	.protonum        =  50,
	.manip_pkt       =  esp_manip_pkt,
	.in_range        =  esp_in_range,
	.unique_tuple    =  esp_unique_tuple,
#if defined(CONFIG_NF_CT_NETLINK) || defined(CONFIG_NF_CT_NETLINK_MODULE)
	.range_to_nlattr = NULL,
	.nlattr_to_range = NULL,
#endif
};
				
static int __init init(void)
{
	if (nf_nat_protocol_register(&esp))
		return -EIO;
		
	printk("Init IP_nat_proto_esp register. \n");
	return 0;
}

static void __exit fini(void)
{
	printk("UnInit IP_nat_proto_esp register. \n");
	nf_nat_protocol_unregister(&esp);
}

module_init(init);
module_exit(fini);

